/*
*************************************************************************
**  Copyright (c) 2019 CentraleSupélec & EDF.
**  All rights reserved. This program and the accompanying materials
**  are made available under the terms of the Eclipse Public License v2.0
**  which accompanies this distribution, and is available at
**  https://www.eclipse.org/legal/epl-v20.html
** 
**  This file is part of the RiseClipse tool
**  
**  Contributors:
**      Computer Science Department, CentraleSupélec
**      EDF R&D
**  Contacts:
**      dominique.marcadet@centralesupelec.fr
**      aurelie.dehouck-neveu@edf.fr
**  Web site:
**      http://wdi.supelec.fr/software/RiseClipse/
*************************************************************************
*/
package fr.centralesupelec.edf.riseclipse.iec61850.scl.provider;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;

import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;

import fr.centralesupelec.edf.riseclipse.iec61850.scl.util.SclAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support Viewers.
 * The adapters generated by this factory convert EMF adapter notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}.
 * The adapters also support Eclipse property sheets.
 * Note that most of the adapters are shared among multiple instances.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class SclItemProviderAdapterFactory extends SclAdapterFactory implements ComposeableAdapterFactory, IChangeNotifier, IDisposable {
    /**
     * This keeps track of the root adapter factory that delegates to this adapter factory.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ComposedAdapterFactory parentAdapterFactory;

    /**
     * This is used to implement {@link org.eclipse.emf.edit.provider.IChangeNotifier}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected IChangeNotifier changeNotifier = new ChangeNotifier();

    /**
     * This keeps track of all the supported types checked by {@link #isFactoryForType isFactoryForType}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected Collection<Object> supportedTypes = new ArrayList<Object>();

    /**
     * This constructs an instance.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public SclItemProviderAdapterFactory() {
        supportedTypes.add(IEditingDomainItemProvider.class);
        supportedTypes.add(IStructuredItemContentProvider.class);
        supportedTypes.add(ITreeItemContentProvider.class);
        supportedTypes.add(IItemLabelProvider.class);
        supportedTypes.add(IItemPropertySource.class);
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Header} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected HeaderItemProvider headerItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Header}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createHeaderAdapter() {
        if (headerItemProvider == null) {
            headerItemProvider = new HeaderItemProvider(this);
        }

        return headerItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.History} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected HistoryItemProvider historyItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.History}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createHistoryAdapter() {
        if (historyItemProvider == null) {
            historyItemProvider = new HistoryItemProvider(this);
        }

        return historyItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Hitem} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected HitemItemProvider hitemItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Hitem}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createHitemAdapter() {
        if (hitemItemProvider == null) {
            hitemItemProvider = new HitemItemProvider(this);
        }

        return hitemItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IDNaming} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected IDNamingItemProvider idNamingItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IDNaming}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createIDNamingAdapter() {
        if (idNamingItemProvider == null) {
            idNamingItemProvider = new IDNamingItemProvider(this);
        }

        return idNamingItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Line} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LineItemProvider lineItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Line}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLineAdapter() {
        if (lineItemProvider == null) {
            lineItemProvider = new LineItemProvider(this);
        }

        return lineItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Private} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected PrivateItemProvider privateItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Private}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createPrivateAdapter() {
        if (privateItemProvider == null) {
            privateItemProvider = new PrivateItemProvider(this);
        }

        return privateItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Process} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ProcessItemProvider processItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Process}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createProcessAdapter() {
        if (processItemProvider == null) {
            processItemProvider = new ProcessItemProvider(this);
        }

        return processItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SCL} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SCLItemProvider sclItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SCL}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSCLAdapter() {
        if (sclItemProvider == null) {
            sclItemProvider = new SCLItemProvider(this);
        }

        return sclItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Text} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TextItemProvider textItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Text}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTextAdapter() {
        if (textItemProvider == null) {
            textItemProvider = new TextItemProvider(this);
        }

        return textItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Address} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AddressItemProvider addressItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Address}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAddressAdapter() {
        if (addressItemProvider == null) {
            addressItemProvider = new AddressItemProvider(this);
        }

        return addressItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Communication} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected CommunicationItemProvider communicationItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Communication}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createCommunicationAdapter() {
        if (communicationItemProvider == null) {
            communicationItemProvider = new CommunicationItemProvider(this);
        }

        return communicationItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConnectedAP} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConnectedAPItemProvider connectedAPItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConnectedAP}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConnectedAPAdapter() {
        if (connectedAPItemProvider == null) {
            connectedAPItemProvider = new ConnectedAPItemProvider(this);
        }

        return connectedAPItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSE} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GSEItemProvider gseItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSE}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGSEAdapter() {
        if (gseItemProvider == null) {
            gseItemProvider = new GSEItemProvider(this);
        }

        return gseItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.P} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected PItemProvider pItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.P}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createPAdapter() {
        if (pItemProvider == null) {
            pItemProvider = new PItemProvider(this);
        }

        return pItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PAddr} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected PAddrItemProvider pAddrItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PAddr}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createPAddrAdapter() {
        if (pAddrItemProvider == null) {
            pAddrItemProvider = new PAddrItemProvider(this);
        }

        return pAddrItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.P_PhysConn} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected P_PhysConnItemProvider p_PhysConnItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.P_PhysConn}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createP_PhysConnAdapter() {
        if (p_PhysConnItemProvider == null) {
            p_PhysConnItemProvider = new P_PhysConnItemProvider(this);
        }

        return p_PhysConnItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PhysConn} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected PhysConnItemProvider physConnItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PhysConn}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createPhysConnAdapter() {
        if (physConnItemProvider == null) {
            physConnItemProvider = new PhysConnItemProvider(this);
        }

        return physConnItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMV} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SMVItemProvider smvItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMV}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSMVAdapter() {
        if (smvItemProvider == null) {
            smvItemProvider = new SMVItemProvider(this);
        }

        return smvItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SubNetwork} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SubNetworkItemProvider subNetworkItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SubNetwork}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSubNetworkAdapter() {
        if (subNetworkItemProvider == null) {
            subNetworkItemProvider = new SubNetworkItemProvider(this);
        }

        return subNetworkItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.BDA} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected BDAItemProvider bdaItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.BDA}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createBDAAdapter() {
        if (bdaItemProvider == null) {
            bdaItemProvider = new BDAItemProvider(this);
        }

        return bdaItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DA} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DAItemProvider daItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DA}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDAAdapter() {
        if (daItemProvider == null) {
            daItemProvider = new DAItemProvider(this);
        }

        return daItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DAType} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DATypeItemProvider daTypeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DAType}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDATypeAdapter() {
        if (daTypeItemProvider == null) {
            daTypeItemProvider = new DATypeItemProvider(this);
        }

        return daTypeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DO} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DOItemProvider doItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DO}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDOAdapter() {
        if (doItemProvider == null) {
            doItemProvider = new DOItemProvider(this);
        }

        return doItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DOType} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DOTypeItemProvider doTypeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DOType}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDOTypeAdapter() {
        if (doTypeItemProvider == null) {
            doTypeItemProvider = new DOTypeItemProvider(this);
        }

        return doTypeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataTypeTemplates} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DataTypeTemplatesItemProvider dataTypeTemplatesItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataTypeTemplates}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDataTypeTemplatesAdapter() {
        if (dataTypeTemplatesItemProvider == null) {
            dataTypeTemplatesItemProvider = new DataTypeTemplatesItemProvider(this);
        }

        return dataTypeTemplatesItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EnumType} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected EnumTypeItemProvider enumTypeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EnumType}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createEnumTypeAdapter() {
        if (enumTypeItemProvider == null) {
            enumTypeItemProvider = new EnumTypeItemProvider(this);
        }

        return enumTypeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EnumVal} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected EnumValItemProvider enumValItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EnumVal}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createEnumValAdapter() {
        if (enumValItemProvider == null) {
            enumValItemProvider = new EnumValItemProvider(this);
        }

        return enumValItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LNodeType} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LNodeTypeItemProvider lNodeTypeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LNodeType}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLNodeTypeAdapter() {
        if (lNodeTypeItemProvider == null) {
            lNodeTypeItemProvider = new LNodeTypeItemProvider(this);
        }

        return lNodeTypeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ProtNs} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ProtNsItemProvider protNsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ProtNs}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createProtNsAdapter() {
        if (protNsItemProvider == null) {
            protNsItemProvider = new ProtNsItemProvider(this);
        }

        return protNsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SDO} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SDOItemProvider sdoItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SDO}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSDOAdapter() {
        if (sdoItemProvider == null) {
            sdoItemProvider = new SDOItemProvider(this);
        }

        return sdoItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Val} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ValItemProvider valItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Val}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createValAdapter() {
        if (valItemProvider == null) {
            valItemProvider = new ValItemProvider(this);
        }

        return valItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.BitRateInMbPerSec} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected BitRateInMbPerSecItemProvider bitRateInMbPerSecItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.BitRateInMbPerSec}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createBitRateInMbPerSecAdapter() {
        if (bitRateInMbPerSecItemProvider == null) {
            bitRateInMbPerSecItemProvider = new BitRateInMbPerSecItemProvider(this);
        }

        return bitRateInMbPerSecItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.MinTime} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected MinTimeItemProvider minTimeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.MinTime}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createMinTimeAdapter() {
        if (minTimeItemProvider == null) {
            minTimeItemProvider = new MinTimeItemProvider(this);
        }

        return minTimeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.MaxTime} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected MaxTimeItemProvider maxTimeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.MaxTime}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createMaxTimeAdapter() {
        if (maxTimeItemProvider == null) {
            maxTimeItemProvider = new MaxTimeItemProvider(this);
        }

        return maxTimeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.BitRate} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected BitRateItemProvider bitRateItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.BitRate}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createBitRateAdapter() {
        if (bitRateItemProvider == null) {
            bitRateItemProvider = new BitRateItemProvider(this);
        }

        return bitRateItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SmpRate} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SmpRateItemProvider smpRateItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SmpRate}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSmpRateAdapter() {
        if (smpRateItemProvider == null) {
            smpRateItemProvider = new SmpRateItemProvider(this);
        }

        return smpRateItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SamplesPerSec} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SamplesPerSecItemProvider samplesPerSecItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SamplesPerSec}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSamplesPerSecAdapter() {
        if (samplesPerSecItemProvider == null) {
            samplesPerSecItemProvider = new SamplesPerSecItemProvider(this);
        }

        return samplesPerSecItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SecPerSamples} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SecPerSamplesItemProvider secPerSamplesItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SecPerSamples}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSecPerSamplesAdapter() {
        if (secPerSamplesItemProvider == null) {
            secPerSamplesItemProvider = new SecPerSamplesItemProvider(this);
        }

        return secPerSamplesItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.McSecurity} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected McSecurityItemProvider mcSecurityItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.McSecurity}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createMcSecurityAdapter() {
        if (mcSecurityItemProvider == null) {
            mcSecurityItemProvider = new McSecurityItemProvider(this);
        }

        return mcSecurityItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AgAuthentication} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AgAuthenticationItemProvider agAuthenticationItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AgAuthentication}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAgAuthenticationAdapter() {
        if (agAuthenticationItemProvider == null) {
            agAuthenticationItemProvider = new AgAuthenticationItemProvider(this);
        }

        return agAuthenticationItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AgLNRef} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AgLNRefItemProvider agLNRefItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AgLNRef}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAgLNRefAdapter() {
        if (agLNRefItemProvider == null) {
            agLNRefItemProvider = new AgLNRefItemProvider(this);
        }

        return agLNRefItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DurationInMilliSec} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DurationInMilliSecItemProvider durationInMilliSecItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DurationInMilliSec}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDurationInMilliSecAdapter() {
        if (durationInMilliSecItemProvider == null) {
            durationInMilliSecItemProvider = new DurationInMilliSecItemProvider(this);
        }

        return durationInMilliSecItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AccessControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AccessControlItemProvider accessControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AccessControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAccessControlAdapter() {
        if (accessControlItemProvider == null) {
            accessControlItemProvider = new AccessControlItemProvider(this);
        }

        return accessControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AccessPoint} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AccessPointItemProvider accessPointItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AccessPoint}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAccessPointAdapter() {
        if (accessPointItemProvider == null) {
            accessPointItemProvider = new AccessPointItemProvider(this);
        }

        return accessPointItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Association} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AssociationItemProvider associationItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Association}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAssociationAdapter() {
        if (associationItemProvider == null) {
            associationItemProvider = new AssociationItemProvider(this);
        }

        return associationItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Authentication} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AuthenticationItemProvider authenticationItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Authentication}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAuthenticationAdapter() {
        if (authenticationItemProvider == null) {
            authenticationItemProvider = new AuthenticationItemProvider(this);
        }

        return authenticationItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Certificate} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected CertificateItemProvider certificateItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Certificate}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createCertificateAdapter() {
        if (certificateItemProvider == null) {
            certificateItemProvider = new CertificateItemProvider(this);
        }

        return certificateItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ClientLN} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ClientLNItemProvider clientLNItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ClientLN}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createClientLNAdapter() {
        if (clientLNItemProvider == null) {
            clientLNItemProvider = new ClientLNItemProvider(this);
        }

        return clientLNItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ClientServices} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ClientServicesItemProvider clientServicesItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ClientServices}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createClientServicesAdapter() {
        if (clientServicesItemProvider == null) {
            clientServicesItemProvider = new ClientServicesItemProvider(this);
        }

        return clientServicesItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.CommProt} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected CommProtItemProvider commProtItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.CommProt}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createCommProtAdapter() {
        if (commProtItemProvider == null) {
            commProtItemProvider = new CommProtItemProvider(this);
        }

        return commProtItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfDataSet} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfDataSetItemProvider confDataSetItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfDataSet}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfDataSetAdapter() {
        if (confDataSetItemProvider == null) {
            confDataSetItemProvider = new ConfDataSetItemProvider(this);
        }

        return confDataSetItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfLNs} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfLNsItemProvider confLNsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfLNs}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfLNsAdapter() {
        if (confLNsItemProvider == null) {
            confLNsItemProvider = new ConfLNsItemProvider(this);
        }

        return confLNsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfLdName} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfLdNameItemProvider confLdNameItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfLdName}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfLdNameAdapter() {
        if (confLdNameItemProvider == null) {
            confLdNameItemProvider = new ConfLdNameItemProvider(this);
        }

        return confLdNameItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfLogControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfLogControlItemProvider confLogControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfLogControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfLogControlAdapter() {
        if (confLogControlItemProvider == null) {
            confLogControlItemProvider = new ConfLogControlItemProvider(this);
        }

        return confLogControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfReportControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfReportControlItemProvider confReportControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfReportControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfReportControlAdapter() {
        if (confReportControlItemProvider == null) {
            confReportControlItemProvider = new ConfReportControlItemProvider(this);
        }

        return confReportControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfSG} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfSGItemProvider confSGItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfSG}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfSGAdapter() {
        if (confSGItemProvider == null) {
            confSGItemProvider = new ConfSGItemProvider(this);
        }

        return confSGItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfSigRef} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConfSigRefItemProvider confSigRefItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConfSigRef}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConfSigRefAdapter() {
        if (confSigRefItemProvider == null) {
            confSigRefItemProvider = new ConfSigRefItemProvider(this);
        }

        return confSigRefItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ControlWithIEDName} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ControlWithIEDNameItemProvider controlWithIEDNameItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ControlWithIEDName}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createControlWithIEDNameAdapter() {
        if (controlWithIEDNameItemProvider == null) {
            controlWithIEDNameItemProvider = new ControlWithIEDNameItemProvider(this);
        }

        return controlWithIEDNameItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DAI} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DAIItemProvider daiItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DAI}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDAIAdapter() {
        if (daiItemProvider == null) {
            daiItemProvider = new DAIItemProvider(this);
        }

        return daiItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DOI} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DOIItemProvider doiItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DOI}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDOIAdapter() {
        if (doiItemProvider == null) {
            doiItemProvider = new DOIItemProvider(this);
        }

        return doiItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataObjectDirectory} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DataObjectDirectoryItemProvider dataObjectDirectoryItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataObjectDirectory}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDataObjectDirectoryAdapter() {
        if (dataObjectDirectoryItemProvider == null) {
            dataObjectDirectoryItemProvider = new DataObjectDirectoryItemProvider(this);
        }

        return dataObjectDirectoryItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataSet} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DataSetItemProvider dataSetItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataSet}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDataSetAdapter() {
        if (dataSetItemProvider == null) {
            dataSetItemProvider = new DataSetItemProvider(this);
        }

        return dataSetItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataSetDirectory} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DataSetDirectoryItemProvider dataSetDirectoryItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DataSetDirectory}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDataSetDirectoryAdapter() {
        if (dataSetDirectoryItemProvider == null) {
            dataSetDirectoryItemProvider = new DataSetDirectoryItemProvider(this);
        }

        return dataSetDirectoryItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DynAssociation} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DynAssociationItemProvider dynAssociationItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DynAssociation}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDynAssociationAdapter() {
        if (dynAssociationItemProvider == null) {
            dynAssociationItemProvider = new DynAssociationItemProvider(this);
        }

        return dynAssociationItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DynDataSet} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DynDataSetItemProvider dynDataSetItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DynDataSet}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDynDataSetAdapter() {
        if (dynDataSetItemProvider == null) {
            dynDataSetItemProvider = new DynDataSetItemProvider(this);
        }

        return dynDataSetItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ExtRef} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ExtRefItemProvider extRefItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ExtRef}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createExtRefAdapter() {
        if (extRefItemProvider == null) {
            extRefItemProvider = new ExtRefItemProvider(this);
        }

        return extRefItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.FCDA} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected FCDAItemProvider fcdaItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.FCDA}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createFCDAAdapter() {
        if (fcdaItemProvider == null) {
            fcdaItemProvider = new FCDAItemProvider(this);
        }

        return fcdaItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.FileHandling} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected FileHandlingItemProvider fileHandlingItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.FileHandling}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createFileHandlingAdapter() {
        if (fileHandlingItemProvider == null) {
            fileHandlingItemProvider = new FileHandlingItemProvider(this);
        }

        return fileHandlingItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GOOSE} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GOOSEItemProvider gooseItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GOOSE}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGOOSEAdapter() {
        if (gooseItemProvider == null) {
            gooseItemProvider = new GOOSEItemProvider(this);
        }

        return gooseItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GOOSESecurity} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GOOSESecurityItemProvider gooseSecurityItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GOOSESecurity}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGOOSESecurityAdapter() {
        if (gooseSecurityItemProvider == null) {
            gooseSecurityItemProvider = new GOOSESecurityItemProvider(this);
        }

        return gooseSecurityItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSEControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GSEControlItemProvider gseControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSEControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGSEControlAdapter() {
        if (gseControlItemProvider == null) {
            gseControlItemProvider = new GSEControlItemProvider(this);
        }

        return gseControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSEDir} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GSEDirItemProvider gseDirItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSEDir}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGSEDirAdapter() {
        if (gseDirItemProvider == null) {
            gseDirItemProvider = new GSEDirItemProvider(this);
        }

        return gseDirItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSESettings} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GSESettingsItemProvider gseSettingsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSESettings}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGSESettingsAdapter() {
        if (gseSettingsItemProvider == null) {
            gseSettingsItemProvider = new GSESettingsItemProvider(this);
        }

        return gseSettingsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSSE} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GSSEItemProvider gsseItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GSSE}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGSSEAdapter() {
        if (gsseItemProvider == null) {
            gsseItemProvider = new GSSEItemProvider(this);
        }

        return gsseItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetCBValues} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GetCBValuesItemProvider getCBValuesItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetCBValues}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGetCBValuesAdapter() {
        if (getCBValuesItemProvider == null) {
            getCBValuesItemProvider = new GetCBValuesItemProvider(this);
        }

        return getCBValuesItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetDataObjectDefinition} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GetDataObjectDefinitionItemProvider getDataObjectDefinitionItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetDataObjectDefinition}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGetDataObjectDefinitionAdapter() {
        if (getDataObjectDefinitionItemProvider == null) {
            getDataObjectDefinitionItemProvider = new GetDataObjectDefinitionItemProvider(this);
        }

        return getDataObjectDefinitionItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetDataSetValue} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GetDataSetValueItemProvider getDataSetValueItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetDataSetValue}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGetDataSetValueAdapter() {
        if (getDataSetValueItemProvider == null) {
            getDataSetValueItemProvider = new GetDataSetValueItemProvider(this);
        }

        return getDataSetValueItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetDirectory} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GetDirectoryItemProvider getDirectoryItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GetDirectory}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGetDirectoryAdapter() {
        if (getDirectoryItemProvider == null) {
            getDirectoryItemProvider = new GetDirectoryItemProvider(this);
        }

        return getDirectoryItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IED} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected IEDItemProvider iedItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IED}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createIEDAdapter() {
        if (iedItemProvider == null) {
            iedItemProvider = new IEDItemProvider(this);
        }

        return iedItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IEDName} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected IEDNameItemProvider iedNameItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IEDName}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createIEDNameAdapter() {
        if (iedNameItemProvider == null) {
            iedNameItemProvider = new IEDNameItemProvider(this);
        }

        return iedNameItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Inputs} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected InputsItemProvider inputsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Inputs}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createInputsAdapter() {
        if (inputsItemProvider == null) {
            inputsItemProvider = new InputsItemProvider(this);
        }

        return inputsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IssuerName} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected IssuerNameItemProvider issuerNameItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.IssuerName}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createIssuerNameAdapter() {
        if (issuerNameItemProvider == null) {
            issuerNameItemProvider = new IssuerNameItemProvider(this);
        }

        return issuerNameItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.KDC} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected KDCItemProvider kdcItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.KDC}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createKDCAdapter() {
        if (kdcItemProvider == null) {
            kdcItemProvider = new KDCItemProvider(this);
        }

        return kdcItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LDevice} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LDeviceItemProvider lDeviceItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LDevice}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLDeviceAdapter() {
        if (lDeviceItemProvider == null) {
            lDeviceItemProvider = new LDeviceItemProvider(this);
        }

        return lDeviceItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LN} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LNItemProvider lnItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LN}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLNAdapter() {
        if (lnItemProvider == null) {
            lnItemProvider = new LNItemProvider(this);
        }

        return lnItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LN0} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LN0ItemProvider ln0ItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LN0}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLN0Adapter() {
        if (ln0ItemProvider == null) {
            ln0ItemProvider = new LN0ItemProvider(this);
        }

        return ln0ItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Log} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LogItemProvider logItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Log}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLogAdapter() {
        if (logItemProvider == null) {
            logItemProvider = new LogItemProvider(this);
        }

        return logItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LogControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LogControlItemProvider logControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LogControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLogControlAdapter() {
        if (logControlItemProvider == null) {
            logControlItemProvider = new LogControlItemProvider(this);
        }

        return logControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LogSettings} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LogSettingsItemProvider logSettingsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LogSettings}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLogSettingsAdapter() {
        if (logSettingsItemProvider == null) {
            logSettingsItemProvider = new LogSettingsItemProvider(this);
        }

        return logSettingsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.OptFields} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected OptFieldsItemProvider optFieldsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.OptFields}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createOptFieldsAdapter() {
        if (optFieldsItemProvider == null) {
            optFieldsItemProvider = new OptFieldsItemProvider(this);
        }

        return optFieldsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Protocol} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ProtocolItemProvider protocolItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Protocol}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createProtocolAdapter() {
        if (protocolItemProvider == null) {
            protocolItemProvider = new ProtocolItemProvider(this);
        }

        return protocolItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ReadWrite} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ReadWriteItemProvider readWriteItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ReadWrite}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createReadWriteAdapter() {
        if (readWriteItemProvider == null) {
            readWriteItemProvider = new ReadWriteItemProvider(this);
        }

        return readWriteItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.RedProt} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected RedProtItemProvider redProtItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.RedProt}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createRedProtAdapter() {
        if (redProtItemProvider == null) {
            redProtItemProvider = new RedProtItemProvider(this);
        }

        return redProtItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ReportControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ReportControlItemProvider reportControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ReportControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createReportControlAdapter() {
        if (reportControlItemProvider == null) {
            reportControlItemProvider = new ReportControlItemProvider(this);
        }

        return reportControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ReportSettings} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ReportSettingsItemProvider reportSettingsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ReportSettings}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createReportSettingsAdapter() {
        if (reportSettingsItemProvider == null) {
            reportSettingsItemProvider = new ReportSettingsItemProvider(this);
        }

        return reportSettingsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.RptEnabled} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected RptEnabledItemProvider rptEnabledItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.RptEnabled}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createRptEnabledAdapter() {
        if (rptEnabledItemProvider == null) {
            rptEnabledItemProvider = new RptEnabledItemProvider(this);
        }

        return rptEnabledItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SDI} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SDIItemProvider sdiItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SDI}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSDIAdapter() {
        if (sdiItemProvider == null) {
            sdiItemProvider = new SDIItemProvider(this);
        }

        return sdiItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SGEdit} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SGEditItemProvider sgEditItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SGEdit}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSGEditAdapter() {
        if (sgEditItemProvider == null) {
            sgEditItemProvider = new SGEditItemProvider(this);
        }

        return sgEditItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMVSecurity} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SMVSecurityItemProvider smvSecurityItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMVSecurity}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSMVSecurityAdapter() {
        if (smvSecurityItemProvider == null) {
            smvSecurityItemProvider = new SMVSecurityItemProvider(this);
        }

        return smvSecurityItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMVSettings} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SMVSettingsItemProvider smvSettingsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMVSettings}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSMVSettingsAdapter() {
        if (smvSettingsItemProvider == null) {
            smvSettingsItemProvider = new SMVSettingsItemProvider(this);
        }

        return smvSettingsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMVsc} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SMVscItemProvider smVscItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SMVsc}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSMVscAdapter() {
        if (smVscItemProvider == null) {
            smVscItemProvider = new SMVscItemProvider(this);
        }

        return smVscItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SampledValueControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SampledValueControlItemProvider sampledValueControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SampledValueControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSampledValueControlAdapter() {
        if (sampledValueControlItemProvider == null) {
            sampledValueControlItemProvider = new SampledValueControlItemProvider(this);
        }

        return sampledValueControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Server} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServerItemProvider serverItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Server}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServerAdapter() {
        if (serverItemProvider == null) {
            serverItemProvider = new ServerItemProvider(this);
        }

        return serverItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServerAt} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServerAtItemProvider serverAtItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServerAt}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServerAtAdapter() {
        if (serverAtItemProvider == null) {
            serverAtItemProvider = new ServerAtItemProvider(this);
        }

        return serverAtItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceSettings} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceSettingsItemProvider serviceSettingsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceSettings}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceSettingsAdapter() {
        if (serviceSettingsItemProvider == null) {
            serviceSettingsItemProvider = new ServiceSettingsItemProvider(this);
        }

        return serviceSettingsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMax} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceWithMaxItemProvider serviceWithMaxItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMax}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceWithMaxAdapter() {
        if (serviceWithMaxItemProvider == null) {
            serviceWithMaxItemProvider = new ServiceWithMaxItemProvider(this);
        }

        return serviceWithMaxItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Services} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServicesItemProvider servicesItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Services}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServicesAdapter() {
        if (servicesItemProvider == null) {
            servicesItemProvider = new ServicesItemProvider(this);
        }

        return servicesItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SetDataSetValue} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SetDataSetValueItemProvider setDataSetValueItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SetDataSetValue}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSetDataSetValueAdapter() {
        if (setDataSetValueItemProvider == null) {
            setDataSetValueItemProvider = new SetDataSetValueItemProvider(this);
        }

        return setDataSetValueItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SettingControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SettingControlItemProvider settingControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SettingControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSettingControlAdapter() {
        if (settingControlItemProvider == null) {
            settingControlItemProvider = new SettingControlItemProvider(this);
        }

        return settingControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SettingGroups} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SettingGroupsItemProvider settingGroupsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SettingGroups}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSettingGroupsAdapter() {
        if (settingGroupsItemProvider == null) {
            settingGroupsItemProvider = new SettingGroupsItemProvider(this);
        }

        return settingGroupsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SmvOpts} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SmvOptsItemProvider smvOptsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SmvOpts}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSmvOptsAdapter() {
        if (smvOptsItemProvider == null) {
            smvOptsItemProvider = new SmvOptsItemProvider(this);
        }

        return smvOptsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Subject} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SubjectItemProvider subjectItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Subject}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSubjectAdapter() {
        if (subjectItemProvider == null) {
            subjectItemProvider = new SubjectItemProvider(this);
        }

        return subjectItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SupSubscription} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SupSubscriptionItemProvider supSubscriptionItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SupSubscription}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSupSubscriptionAdapter() {
        if (supSubscriptionItemProvider == null) {
            supSubscriptionItemProvider = new SupSubscriptionItemProvider(this);
        }

        return supSubscriptionItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TimeSyncProt} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TimeSyncProtItemProvider timeSyncProtItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TimeSyncProt}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTimeSyncProtAdapter() {
        if (timeSyncProtItemProvider == null) {
            timeSyncProtItemProvider = new TimeSyncProtItemProvider(this);
        }

        return timeSyncProtItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TimerActivatedControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TimerActivatedControlItemProvider timerActivatedControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TimerActivatedControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTimerActivatedControlAdapter() {
        if (timerActivatedControlItemProvider == null) {
            timerActivatedControlItemProvider = new TimerActivatedControlItemProvider(this);
        }

        return timerActivatedControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TrgOps} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TrgOpsItemProvider trgOpsItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TrgOps}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTrgOpsAdapter() {
        if (trgOpsItemProvider == null) {
            trgOpsItemProvider = new TrgOpsItemProvider(this);
        }

        return trgOpsItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ValueHandling} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ValueHandlingItemProvider valueHandlingItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ValueHandling}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createValueHandlingAdapter() {
        if (valueHandlingItemProvider == null) {
            valueHandlingItemProvider = new ValueHandlingItemProvider(this);
        }

        return valueHandlingItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AbstractConductingEquipment} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AbstractConductingEquipmentItemProvider abstractConductingEquipmentItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AbstractConductingEquipment}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAbstractConductingEquipmentAdapter() {
        if (abstractConductingEquipmentItemProvider == null) {
            abstractConductingEquipmentItemProvider = new AbstractConductingEquipmentItemProvider(this);
        }

        return abstractConductingEquipmentItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AbstractEqFuncSubFunc} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected AbstractEqFuncSubFuncItemProvider abstractEqFuncSubFuncItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.AbstractEqFuncSubFunc}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createAbstractEqFuncSubFuncAdapter() {
        if (abstractEqFuncSubFuncItemProvider == null) {
            abstractEqFuncSubFuncItemProvider = new AbstractEqFuncSubFuncItemProvider(this);
        }

        return abstractEqFuncSubFuncItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Bay} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected BayItemProvider bayItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Bay}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createBayAdapter() {
        if (bayItemProvider == null) {
            bayItemProvider = new BayItemProvider(this);
        }

        return bayItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConductingEquipment} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConductingEquipmentItemProvider conductingEquipmentItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConductingEquipment}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConductingEquipmentAdapter() {
        if (conductingEquipmentItemProvider == null) {
            conductingEquipmentItemProvider = new ConductingEquipmentItemProvider(this);
        }

        return conductingEquipmentItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConnectivityNode} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ConnectivityNodeItemProvider connectivityNodeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ConnectivityNode}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createConnectivityNodeAdapter() {
        if (connectivityNodeItemProvider == null) {
            connectivityNodeItemProvider = new ConnectivityNodeItemProvider(this);
        }

        return connectivityNodeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EqFunction} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected EqFunctionItemProvider eqFunctionItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EqFunction}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createEqFunctionAdapter() {
        if (eqFunctionItemProvider == null) {
            eqFunctionItemProvider = new EqFunctionItemProvider(this);
        }

        return eqFunctionItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EqSubFunction} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected EqSubFunctionItemProvider eqSubFunctionItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EqSubFunction}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createEqSubFunctionAdapter() {
        if (eqSubFunctionItemProvider == null) {
            eqSubFunctionItemProvider = new EqSubFunctionItemProvider(this);
        }

        return eqSubFunctionItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Equipment} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected EquipmentItemProvider equipmentItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Equipment}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createEquipmentAdapter() {
        if (equipmentItemProvider == null) {
            equipmentItemProvider = new EquipmentItemProvider(this);
        }

        return equipmentItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EquipmentContainer} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected EquipmentContainerItemProvider equipmentContainerItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.EquipmentContainer}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createEquipmentContainerAdapter() {
        if (equipmentContainerItemProvider == null) {
            equipmentContainerItemProvider = new EquipmentContainerItemProvider(this);
        }

        return equipmentContainerItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Function} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected FunctionItemProvider functionItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Function}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createFunctionAdapter() {
        if (functionItemProvider == null) {
            functionItemProvider = new FunctionItemProvider(this);
        }

        return functionItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GeneralEquipment} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GeneralEquipmentItemProvider generalEquipmentItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GeneralEquipment}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGeneralEquipmentAdapter() {
        if (generalEquipmentItemProvider == null) {
            generalEquipmentItemProvider = new GeneralEquipmentItemProvider(this);
        }

        return generalEquipmentItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GeneralEquipmentContainer} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected GeneralEquipmentContainerItemProvider generalEquipmentContainerItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.GeneralEquipmentContainer}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createGeneralEquipmentContainerAdapter() {
        if (generalEquipmentContainerItemProvider == null) {
            generalEquipmentContainerItemProvider = new GeneralEquipmentContainerItemProvider(this);
        }

        return generalEquipmentContainerItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LNode} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LNodeItemProvider lNodeItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LNode}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLNodeAdapter() {
        if (lNodeItemProvider == null) {
            lNodeItemProvider = new LNodeItemProvider(this);
        }

        return lNodeItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LNodeContainer} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected LNodeContainerItemProvider lNodeContainerItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.LNodeContainer}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createLNodeContainerAdapter() {
        if (lNodeContainerItemProvider == null) {
            lNodeContainerItemProvider = new LNodeContainerItemProvider(this);
        }

        return lNodeContainerItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.NeutralPoint} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected NeutralPointItemProvider neutralPointItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.NeutralPoint}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createNeutralPointAdapter() {
        if (neutralPointItemProvider == null) {
            neutralPointItemProvider = new NeutralPointItemProvider(this);
        }

        return neutralPointItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PowerSystemResource} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected PowerSystemResourceItemProvider powerSystemResourceItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PowerSystemResource}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createPowerSystemResourceAdapter() {
        if (powerSystemResourceItemProvider == null) {
            powerSystemResourceItemProvider = new PowerSystemResourceItemProvider(this);
        }

        return powerSystemResourceItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PowerTransformer} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected PowerTransformerItemProvider powerTransformerItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.PowerTransformer}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createPowerTransformerAdapter() {
        if (powerTransformerItemProvider == null) {
            powerTransformerItemProvider = new PowerTransformerItemProvider(this);
        }

        return powerTransformerItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SubEquipment} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SubEquipmentItemProvider subEquipmentItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SubEquipment}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSubEquipmentAdapter() {
        if (subEquipmentItemProvider == null) {
            subEquipmentItemProvider = new SubEquipmentItemProvider(this);
        }

        return subEquipmentItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SubFunction} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SubFunctionItemProvider subFunctionItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.SubFunction}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSubFunctionAdapter() {
        if (subFunctionItemProvider == null) {
            subFunctionItemProvider = new SubFunctionItemProvider(this);
        }

        return subFunctionItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Substation} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected SubstationItemProvider substationItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Substation}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createSubstationAdapter() {
        if (substationItemProvider == null) {
            substationItemProvider = new SubstationItemProvider(this);
        }

        return substationItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TapChanger} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TapChangerItemProvider tapChangerItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TapChanger}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTapChangerAdapter() {
        if (tapChangerItemProvider == null) {
            tapChangerItemProvider = new TapChangerItemProvider(this);
        }

        return tapChangerItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Terminal} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TerminalItemProvider terminalItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Terminal}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTerminalAdapter() {
        if (terminalItemProvider == null) {
            terminalItemProvider = new TerminalItemProvider(this);
        }

        return terminalItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TransformerWinding} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected TransformerWindingItemProvider transformerWindingItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.TransformerWinding}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createTransformerWindingAdapter() {
        if (transformerWindingItemProvider == null) {
            transformerWindingItemProvider = new TransformerWindingItemProvider(this);
        }

        return transformerWindingItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Voltage} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected VoltageItemProvider voltageItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Voltage}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createVoltageAdapter() {
        if (voltageItemProvider == null) {
            voltageItemProvider = new VoltageItemProvider(this);
        }

        return voltageItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.VoltageLevel} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected VoltageLevelItemProvider voltageLevelItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.VoltageLevel}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createVoltageLevelAdapter() {
        if (voltageLevelItemProvider == null) {
            voltageLevelItemProvider = new VoltageLevelItemProvider(this);
        }

        return voltageLevelItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceYesNo} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceYesNoItemProvider serviceYesNoItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceYesNo}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceYesNoAdapter() {
        if (serviceYesNoItemProvider == null) {
            serviceYesNoItemProvider = new ServiceYesNoItemProvider(this);
        }

        return serviceYesNoItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithOptionalMax} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceWithOptionalMaxItemProvider serviceWithOptionalMaxItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithOptionalMax}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceWithOptionalMaxAdapter() {
        if (serviceWithOptionalMaxItemProvider == null) {
            serviceWithOptionalMaxItemProvider = new ServiceWithOptionalMaxItemProvider(this);
        }

        return serviceWithOptionalMaxItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMaxNonZero} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceWithMaxNonZeroItemProvider serviceWithMaxNonZeroItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMaxNonZero}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceWithMaxNonZeroAdapter() {
        if (serviceWithMaxNonZeroItemProvider == null) {
            serviceWithMaxNonZeroItemProvider = new ServiceWithMaxNonZeroItemProvider(this);
        }

        return serviceWithMaxNonZeroItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceConfReportControl} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceConfReportControlItemProvider serviceConfReportControlItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceConfReportControl}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceConfReportControlAdapter() {
        if (serviceConfReportControlItemProvider == null) {
            serviceConfReportControlItemProvider = new ServiceConfReportControlItemProvider(this);
        }

        return serviceConfReportControlItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMaxAndMaxAttributes} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceWithMaxAndMaxAttributesItemProvider serviceWithMaxAndMaxAttributesItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMaxAndMaxAttributes}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceWithMaxAndMaxAttributesAdapter() {
        if (serviceWithMaxAndMaxAttributesItemProvider == null) {
            serviceWithMaxAndMaxAttributesItemProvider = new ServiceWithMaxAndMaxAttributesItemProvider(this);
        }

        return serviceWithMaxAndMaxAttributesItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMaxAndModify} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceWithMaxAndModifyItemProvider serviceWithMaxAndModifyItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceWithMaxAndModify}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceWithMaxAndModifyAdapter() {
        if (serviceWithMaxAndModifyItemProvider == null) {
            serviceWithMaxAndModifyItemProvider = new ServiceWithMaxAndModifyItemProvider(this);
        }

        return serviceWithMaxAndModifyItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceForConfDataSet} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ServiceForConfDataSetItemProvider serviceForConfDataSetItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ServiceForConfDataSet}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createServiceForConfDataSetAdapter() {
        if (serviceForConfDataSetItemProvider == null) {
            serviceForConfDataSetItemProvider = new ServiceForConfDataSetItemProvider(this);
        }

        return serviceForConfDataSetItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Cert} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected CertItemProvider certItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.Cert}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createCertAdapter() {
        if (certItemProvider == null) {
            certItemProvider = new CertItemProvider(this);
        }

        return certItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ValueWithUnit} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected ValueWithUnitItemProvider valueWithUnitItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.ValueWithUnit}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createValueWithUnitAdapter() {
        if (valueWithUnitItemProvider == null) {
            valueWithUnitItemProvider = new ValueWithUnitItemProvider(this);
        }

        return valueWithUnitItemProvider;
    }

    /**
     * This keeps track of the one adapter used for all {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DurationInSec} instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    protected DurationInSecItemProvider durationInSecItemProvider;

    /**
     * This creates an adapter for a {@link fr.centralesupelec.edf.riseclipse.iec61850.scl.DurationInSec}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter createDurationInSecAdapter() {
        if (durationInSecItemProvider == null) {
            durationInSecItemProvider = new DurationInSecItemProvider(this);
        }

        return durationInSecItemProvider;
    }

    /**
     * This returns the root adapter factory that contains this factory.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public ComposeableAdapterFactory getRootAdapterFactory() {
        return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
    }

    /**
     * This sets the composed adapter factory that contains this factory.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public void setParentAdapterFactory(ComposedAdapterFactory parentAdapterFactory) {
        this.parentAdapterFactory = parentAdapterFactory;
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public boolean isFactoryForType(Object type) {
        return supportedTypes.contains(type) || super.isFactoryForType(type);
    }

    /**
     * This implementation substitutes the factory itself as the key for the adapter.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Adapter adapt(Notifier notifier, Object type) {
        return super.adapt(notifier, this);
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public Object adapt(Object object, Object type) {
        if (isFactoryForType(type)) {
            Object adapter = super.adapt(object, type);
            if (!(type instanceof Class<?>) || (((Class<?>)type).isInstance(adapter))) {
                return adapter;
            }
        }

        return null;
    }

    /**
     * This adds a listener.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public void addListener(INotifyChangedListener notifyChangedListener) {
        changeNotifier.addListener(notifyChangedListener);
    }

    /**
     * This removes a listener.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public void removeListener(INotifyChangedListener notifyChangedListener) {
        changeNotifier.removeListener(notifyChangedListener);
    }

    /**
     * This delegates to {@link #changeNotifier} and to {@link #parentAdapterFactory}.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public void fireNotifyChanged(Notification notification) {
        changeNotifier.fireNotifyChanged(notification);

        if (parentAdapterFactory != null) {
            parentAdapterFactory.fireNotifyChanged(notification);
        }
    }

    /**
     * This disposes all of the item providers created by this factory. 
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    @Override
    public void dispose() {
        if (headerItemProvider != null) headerItemProvider.dispose();
        if (historyItemProvider != null) historyItemProvider.dispose();
        if (hitemItemProvider != null) hitemItemProvider.dispose();
        if (idNamingItemProvider != null) idNamingItemProvider.dispose();
        if (lineItemProvider != null) lineItemProvider.dispose();
        if (privateItemProvider != null) privateItemProvider.dispose();
        if (processItemProvider != null) processItemProvider.dispose();
        if (sclItemProvider != null) sclItemProvider.dispose();
        if (textItemProvider != null) textItemProvider.dispose();
        if (addressItemProvider != null) addressItemProvider.dispose();
        if (communicationItemProvider != null) communicationItemProvider.dispose();
        if (connectedAPItemProvider != null) connectedAPItemProvider.dispose();
        if (gseItemProvider != null) gseItemProvider.dispose();
        if (pItemProvider != null) pItemProvider.dispose();
        if (pAddrItemProvider != null) pAddrItemProvider.dispose();
        if (p_PhysConnItemProvider != null) p_PhysConnItemProvider.dispose();
        if (physConnItemProvider != null) physConnItemProvider.dispose();
        if (smvItemProvider != null) smvItemProvider.dispose();
        if (subNetworkItemProvider != null) subNetworkItemProvider.dispose();
        if (bdaItemProvider != null) bdaItemProvider.dispose();
        if (daItemProvider != null) daItemProvider.dispose();
        if (daTypeItemProvider != null) daTypeItemProvider.dispose();
        if (doItemProvider != null) doItemProvider.dispose();
        if (doTypeItemProvider != null) doTypeItemProvider.dispose();
        if (dataTypeTemplatesItemProvider != null) dataTypeTemplatesItemProvider.dispose();
        if (enumTypeItemProvider != null) enumTypeItemProvider.dispose();
        if (enumValItemProvider != null) enumValItemProvider.dispose();
        if (lNodeTypeItemProvider != null) lNodeTypeItemProvider.dispose();
        if (protNsItemProvider != null) protNsItemProvider.dispose();
        if (sdoItemProvider != null) sdoItemProvider.dispose();
        if (valItemProvider != null) valItemProvider.dispose();
        if (accessControlItemProvider != null) accessControlItemProvider.dispose();
        if (accessPointItemProvider != null) accessPointItemProvider.dispose();
        if (associationItemProvider != null) associationItemProvider.dispose();
        if (authenticationItemProvider != null) authenticationItemProvider.dispose();
        if (certificateItemProvider != null) certificateItemProvider.dispose();
        if (clientLNItemProvider != null) clientLNItemProvider.dispose();
        if (clientServicesItemProvider != null) clientServicesItemProvider.dispose();
        if (commProtItemProvider != null) commProtItemProvider.dispose();
        if (confDataSetItemProvider != null) confDataSetItemProvider.dispose();
        if (confLNsItemProvider != null) confLNsItemProvider.dispose();
        if (confLdNameItemProvider != null) confLdNameItemProvider.dispose();
        if (confLogControlItemProvider != null) confLogControlItemProvider.dispose();
        if (confReportControlItemProvider != null) confReportControlItemProvider.dispose();
        if (confSGItemProvider != null) confSGItemProvider.dispose();
        if (confSigRefItemProvider != null) confSigRefItemProvider.dispose();
        if (controlWithIEDNameItemProvider != null) controlWithIEDNameItemProvider.dispose();
        if (daiItemProvider != null) daiItemProvider.dispose();
        if (doiItemProvider != null) doiItemProvider.dispose();
        if (dataObjectDirectoryItemProvider != null) dataObjectDirectoryItemProvider.dispose();
        if (dataSetItemProvider != null) dataSetItemProvider.dispose();
        if (dataSetDirectoryItemProvider != null) dataSetDirectoryItemProvider.dispose();
        if (dynAssociationItemProvider != null) dynAssociationItemProvider.dispose();
        if (dynDataSetItemProvider != null) dynDataSetItemProvider.dispose();
        if (extRefItemProvider != null) extRefItemProvider.dispose();
        if (fcdaItemProvider != null) fcdaItemProvider.dispose();
        if (fileHandlingItemProvider != null) fileHandlingItemProvider.dispose();
        if (gooseItemProvider != null) gooseItemProvider.dispose();
        if (gooseSecurityItemProvider != null) gooseSecurityItemProvider.dispose();
        if (gseControlItemProvider != null) gseControlItemProvider.dispose();
        if (gseDirItemProvider != null) gseDirItemProvider.dispose();
        if (gseSettingsItemProvider != null) gseSettingsItemProvider.dispose();
        if (gsseItemProvider != null) gsseItemProvider.dispose();
        if (getCBValuesItemProvider != null) getCBValuesItemProvider.dispose();
        if (getDataObjectDefinitionItemProvider != null) getDataObjectDefinitionItemProvider.dispose();
        if (getDataSetValueItemProvider != null) getDataSetValueItemProvider.dispose();
        if (getDirectoryItemProvider != null) getDirectoryItemProvider.dispose();
        if (iedItemProvider != null) iedItemProvider.dispose();
        if (iedNameItemProvider != null) iedNameItemProvider.dispose();
        if (inputsItemProvider != null) inputsItemProvider.dispose();
        if (issuerNameItemProvider != null) issuerNameItemProvider.dispose();
        if (kdcItemProvider != null) kdcItemProvider.dispose();
        if (lDeviceItemProvider != null) lDeviceItemProvider.dispose();
        if (lnItemProvider != null) lnItemProvider.dispose();
        if (ln0ItemProvider != null) ln0ItemProvider.dispose();
        if (logItemProvider != null) logItemProvider.dispose();
        if (logControlItemProvider != null) logControlItemProvider.dispose();
        if (logSettingsItemProvider != null) logSettingsItemProvider.dispose();
        if (optFieldsItemProvider != null) optFieldsItemProvider.dispose();
        if (protocolItemProvider != null) protocolItemProvider.dispose();
        if (readWriteItemProvider != null) readWriteItemProvider.dispose();
        if (redProtItemProvider != null) redProtItemProvider.dispose();
        if (reportControlItemProvider != null) reportControlItemProvider.dispose();
        if (reportSettingsItemProvider != null) reportSettingsItemProvider.dispose();
        if (rptEnabledItemProvider != null) rptEnabledItemProvider.dispose();
        if (sdiItemProvider != null) sdiItemProvider.dispose();
        if (sgEditItemProvider != null) sgEditItemProvider.dispose();
        if (smvSecurityItemProvider != null) smvSecurityItemProvider.dispose();
        if (smvSettingsItemProvider != null) smvSettingsItemProvider.dispose();
        if (smVscItemProvider != null) smVscItemProvider.dispose();
        if (sampledValueControlItemProvider != null) sampledValueControlItemProvider.dispose();
        if (serverItemProvider != null) serverItemProvider.dispose();
        if (serverAtItemProvider != null) serverAtItemProvider.dispose();
        if (serviceSettingsItemProvider != null) serviceSettingsItemProvider.dispose();
        if (serviceWithMaxItemProvider != null) serviceWithMaxItemProvider.dispose();
        if (servicesItemProvider != null) servicesItemProvider.dispose();
        if (setDataSetValueItemProvider != null) setDataSetValueItemProvider.dispose();
        if (settingControlItemProvider != null) settingControlItemProvider.dispose();
        if (settingGroupsItemProvider != null) settingGroupsItemProvider.dispose();
        if (smvOptsItemProvider != null) smvOptsItemProvider.dispose();
        if (subjectItemProvider != null) subjectItemProvider.dispose();
        if (supSubscriptionItemProvider != null) supSubscriptionItemProvider.dispose();
        if (timeSyncProtItemProvider != null) timeSyncProtItemProvider.dispose();
        if (timerActivatedControlItemProvider != null) timerActivatedControlItemProvider.dispose();
        if (trgOpsItemProvider != null) trgOpsItemProvider.dispose();
        if (valueHandlingItemProvider != null) valueHandlingItemProvider.dispose();
        if (abstractConductingEquipmentItemProvider != null) abstractConductingEquipmentItemProvider.dispose();
        if (abstractEqFuncSubFuncItemProvider != null) abstractEqFuncSubFuncItemProvider.dispose();
        if (bayItemProvider != null) bayItemProvider.dispose();
        if (conductingEquipmentItemProvider != null) conductingEquipmentItemProvider.dispose();
        if (connectivityNodeItemProvider != null) connectivityNodeItemProvider.dispose();
        if (eqFunctionItemProvider != null) eqFunctionItemProvider.dispose();
        if (eqSubFunctionItemProvider != null) eqSubFunctionItemProvider.dispose();
        if (equipmentItemProvider != null) equipmentItemProvider.dispose();
        if (equipmentContainerItemProvider != null) equipmentContainerItemProvider.dispose();
        if (functionItemProvider != null) functionItemProvider.dispose();
        if (generalEquipmentItemProvider != null) generalEquipmentItemProvider.dispose();
        if (generalEquipmentContainerItemProvider != null) generalEquipmentContainerItemProvider.dispose();
        if (lNodeItemProvider != null) lNodeItemProvider.dispose();
        if (lNodeContainerItemProvider != null) lNodeContainerItemProvider.dispose();
        if (neutralPointItemProvider != null) neutralPointItemProvider.dispose();
        if (powerSystemResourceItemProvider != null) powerSystemResourceItemProvider.dispose();
        if (powerTransformerItemProvider != null) powerTransformerItemProvider.dispose();
        if (subEquipmentItemProvider != null) subEquipmentItemProvider.dispose();
        if (subFunctionItemProvider != null) subFunctionItemProvider.dispose();
        if (substationItemProvider != null) substationItemProvider.dispose();
        if (tapChangerItemProvider != null) tapChangerItemProvider.dispose();
        if (terminalItemProvider != null) terminalItemProvider.dispose();
        if (transformerWindingItemProvider != null) transformerWindingItemProvider.dispose();
        if (voltageItemProvider != null) voltageItemProvider.dispose();
        if (voltageLevelItemProvider != null) voltageLevelItemProvider.dispose();
        if (serviceYesNoItemProvider != null) serviceYesNoItemProvider.dispose();
        if (serviceWithOptionalMaxItemProvider != null) serviceWithOptionalMaxItemProvider.dispose();
        if (serviceWithMaxNonZeroItemProvider != null) serviceWithMaxNonZeroItemProvider.dispose();
        if (serviceConfReportControlItemProvider != null) serviceConfReportControlItemProvider.dispose();
        if (serviceWithMaxAndMaxAttributesItemProvider != null) serviceWithMaxAndMaxAttributesItemProvider.dispose();
        if (serviceWithMaxAndModifyItemProvider != null) serviceWithMaxAndModifyItemProvider.dispose();
        if (serviceForConfDataSetItemProvider != null) serviceForConfDataSetItemProvider.dispose();
        if (certItemProvider != null) certItemProvider.dispose();
        if (valueWithUnitItemProvider != null) valueWithUnitItemProvider.dispose();
        if (durationInSecItemProvider != null) durationInSecItemProvider.dispose();
        if (durationInMilliSecItemProvider != null) durationInMilliSecItemProvider.dispose();
        if (bitRateInMbPerSecItemProvider != null) bitRateInMbPerSecItemProvider.dispose();
        if (minTimeItemProvider != null) minTimeItemProvider.dispose();
        if (maxTimeItemProvider != null) maxTimeItemProvider.dispose();
        if (bitRateItemProvider != null) bitRateItemProvider.dispose();
        if (smpRateItemProvider != null) smpRateItemProvider.dispose();
        if (samplesPerSecItemProvider != null) samplesPerSecItemProvider.dispose();
        if (secPerSamplesItemProvider != null) secPerSamplesItemProvider.dispose();
        if (mcSecurityItemProvider != null) mcSecurityItemProvider.dispose();
        if (agAuthenticationItemProvider != null) agAuthenticationItemProvider.dispose();
        if (agLNRefItemProvider != null) agLNRefItemProvider.dispose();
    }

}
